----------------------------------------------------------------------------------
--  
--        cic_3_4096.vhd        bhbtB^[( N=3, R=4096 )
--  
--        (c)2010/01/11  by OjisanKoubou
--  
----------------------------------------------------------------------------------
--  
--  ̃vO Ojisankoubou( OGAWA Ichiro ) 쌠Ă܂B
--  ľړIɂ́ACAȂǎRɎgpł܂B
--  \ۂ͒쌠\Ȃ悤ɂĂB
--  ̃\tgEFȂS͈ꕔ𐻕iɑgݍ񂾂̔肷ȂǏƓIp͂߂B
--
--20100111 N=3, R=512
-- Number of Slices:                      269  out of   2448    10%  
-- Number of Slice Flip Flops:            451  out of   4896     9%  
-- Number of 4 input LUTs:                292  out of   4896     5%  
-- Number of IOs:                          39
-- Number of bonded IOBs:                  39  out of     66    59%  
-- Number of GCLKs:                         1  out of     24     4%  
--
--   Minimum period: 5.243ns (Maximum Frequency: 190.714MHz)
--   Minimum input arrival time before clock: 3.743ns
--   Maximum output required time after clock: 4.825ns
--   Maximum combinational path delay: No path found

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

------------------------------------------------------------------------------
entity cic_3_4096 is 
  generic (
    DIN_SIZE  : INTEGER := 18;  -- ̓rbg
    DATA_SIZE : INTEGER := 54;  -- f[^rbgiCICiƍőfV[VŕKv
                                -- rbg܂j = DIN_SIZE + N * log2( RMAX )
    DOUT_SIZE : INTEGER := 18;  -- o̓rbg
    N         : INTEGER := 3;   -- CIC i
    RMAX      : INTEGER := 4096  -- őfV[V
  );
  port (
    CICR_RESET   : in  std_logic;
    CICR_CLK     : in  std_logic;
    CICR_DATAIN  : in  std_logic_vector( DIN_SIZE-1 downto 0);  -- ̓f[^
    CICR_WEN     : out std_logic;                               -- o̓f[^݃^C~O
    CICR_DATAOUT : out std_logic_vector( DOUT_SIZE-1 downto 0)  -- o̓f[^
  );
end cic_3_4096;


------------------------------------------------------------------------------
architecture RTL of cic_3_4096 is
  
  type darray is array (N downto 0) of std_logic_vector(DATA_SIZE-1 downto 0);
  signal D     :  darray;        -- Integrater 
  signal DR    :  darray;        -- Comb Filter 
  signal DRM1  :  darray;        -- Comb Filter p̒x
  
  signal R      : integer range 0 to RMAX-1; -- fV[V
  signal RCOUNT : integer range 0 to RMAX-1; -- fV[ṼJE^[
  signal WEN    : std_logic;                 -- o̓f[^L
  
  signal DATAOUT: std_logic_vector(18 downto 0);
  
  begin
    
    CICR_WEN <= WEN;
    
    R <= RMAX-1;
    
    DATAOUT <= DR(N)( DATA_SIZE-1 downto DATA_SIZE-DOUT_SIZE-1 );
    
    
    ------------------------------------------------------------------
    -- ŏIrbgÔPďo 
    ------------------------------------------------------------------
    process( CICR_RESET, CICR_CLK ) begin
      if (rising_edge (CICR_CLK)) then
        CICR_DATAOUT <= DATAOUT(18 downto 1 ) + ("00000000000000000" & DATAOUT(0) );    --
      end if;
    end PROCESS;
    
    
    ------------------------------------------------------------------
    -- Integrater   ͐M signed 
    ------------------------------------------------------------------
    process( CICR_RESET, CICR_CLK ) begin
      if (CICR_RESET = '1') then 
        for i in 0 to N loop
          D(i) <= (others => '0');
        end loop;
      elsif (rising_edge (CICR_CLK)) then
          D(0)(DATA_SIZE-1 downto DIN_SIZE) <= (others => CICR_DATAIN(DIN_SIZE-1)); -- g
          D(0)(DIN_SIZE-1 downto 0) <= CICR_DATAIN;
          for i in 1 to N loop          -- integration loop
            D(i) <= D(i-1) + D(i);
          end loop;
      end if;
    end PROCESS;
    
    
    ------------------------------------------------------------------
    --  Comb Filter
    ------------------------------------------------------------------
    process( CICR_RESET, CICR_CLK ) begin
      if (CICR_RESET = '1') then 
        DR(0) <= (others => '0');
        for i in 1 to N loop
          DRM1(i) <= (others => '0');
          DR(i) <= (others => '0');
        end loop;
      elsif (rising_edge (CICR_CLK)) then
        if (WEN = '1') then             -- decimation
          DR(0) <= D(N); 
          for i in 1 to N loop
            DRM1(i) <= DR(i-1);
            DR(i) <= DR(i-1) - DRM1(i);
          end loop;
        end if;
      end if;
    end PROCESS;
    
    
    ------------------------------------------------------------------
    --  Decimation rate counter
    ------------------------------------------------------------------
    process( CICR_RESET, CICR_CLK ) begin
      if( CICR_RESET = '1' ) then
        RCOUNT <= 0;
      elsif( rising_edge( CICR_CLK ) ) then
          if( RCOUNT = 0 ) then
            WEN <= '1';
            RCOUNT <= R;
          else
            WEN <= '0';
            RCOUNT <= RCOUNT - 1;
          end if;
      end if;
    end process;
    
    
  end RTL;
----------------------------- end of architecture ----------------------------

